@@ -77,6 +77,11 @@
   (declare (ignore args database))
   "DATETIME")
 
+(defmethod database-get-type-specifier ((type (eql 'date)) args database
+                                        (db-type (eql :mssql)))
+  (declare (ignore args database))
+  "SMALLDATETIME")
+
 (defmethod database-get-type-specifier ((type (eql 'boolean)) args database
                                         (db-type (eql :mssql)))
   (declare (ignore args database))
